%global scl oracle-armtoolset-8
%global _scl_prefix /opt/oracle
%{?scl:%{?scl_package:%scl_package binutils}}

%define enable_lto 1
%define enable_shared 1
%define default_compress_debug 0
%define enable_deterministic_archives 0
%define binutils_target %{_target_platform}

Summary: A GNU collection of binary utilities
Name: %{?scl_prefix}binutils
Version: 2.31.1
Release: 1%{?dist}
License: GPLv3+
Group: Development/Tools
URL: http://www.gnu.org/s/binutils

%define basesourcename binutils-%{version}

Source: http://ftp.gnu.org/gnu/binutils/%{basesourcename}.tar.bz2
Source1: binutils-2.19.50.0.1-output-format.sed

Patch01: binutils-testsuite-aarch64-ifunc.patch
#Orabug 28182242:
Patch02: binutils-discarded-dynsym.patch
Provides: bundled(libiberty)

# Determine whether building ld/gold or not.

%define gold_arches %ix86 x86_64 %arm aarch64

%ifarch %gold_arches
%define enable_gold		yes
%define enable_ld		default
%define ld_bfd_priority		50
%define ld_gold_priority	30
%else
%define enable_gold		no
%define enable_ld		default
%define ld_bfd_priority		50
%define ld_gold_priority	0
%endif

%define alternatives_cmd %{!?scl:%{_sbindir}}%{?scl:%{_root_sbindir}}/alternatives
%define alternatives_cmdline %{alternatives_cmd}%{?scl: --altdir %{_sysconfdir}/alternatives --admindir %{_scl_root}/var/lib/alternatives}

Buildroot: %(mktemp -ud %{_tmppath}/%{name}-%{version}-%{release}-XXXXXX)

# Build-time requirements.

BuildRequires: gcc
# Gold needs bison in order to build gold/yyscript.c.
# Bison needs m4.
%if "%{enable_gold}" != "no"
BuildRequires: bison, m4, gcc-c++
%endif
BuildRequires: gettext, flex, zlib-devel
BuildRequires: texinfo >= 4.0
BuildRequires: /usr/bin/pod2man
BuildRequires: dejagnu, zlib-static, glibc-static, sharutils, bc
BuildRequires: libstdc++-static

Conflicts: gcc-c++ < 4.0.0

# Run-time requirements.

%{?scl:Requires:%scl_runtime}

Requires(post): /sbin/install-info
Requires(preun): /sbin/install-info

%if "%{enable_gold}" != "no"
Requires(post): coreutils
Requires(post): %{alternatives_cmd}
Requires(preun): %{alternatives_cmd}
%endif

# On ARM EABI systems, we do want -gnueabi to be part of the
# target triple.
%ifnarch %{arm}
%define _gnu %{nil}
%endif

%description
Binutils is a collection of binary utilities, including ar (for
creating, modifying and extracting from archives), as (a family of GNU
assemblers), gprof (for displaying call graph profile data), ld (the
GNU linker), nm (for listing symbols from object files), objcopy (for
copying and translating object files), objdump (for displaying
information from object files), ranlib (for generating an index for
the contents of an archive), readelf (for displaying detailed
information about binary files), size (for listing the section sizes
of an object or archive file), strings (for listing printable strings
from files), strip (for discarding symbols), and addr2line (for
converting addresses to file and line).

%package devel
Summary: BFD and opcodes static and dynamic libraries and header files
Group: System Environment/Libraries
Provides: binutils-static = %{version}-%{release}
%if %{with docs}
Requires(post): /sbin/install-info
Requires(preun): /sbin/install-info
%endif
Requires: zlib-devel
Requires: %{?scl_prefix}binutils = %{version}-%{release}
Requires: coreutils

%description devel
This package contains BFD and opcodes static and dynamic libraries.

The dynamic libraries are in this package, rather than a seperate
base package because they are actually linker scripts that force
the use of the static libraries.  This is because the API of the
BFD library is too unstable to be used dynamically.

The static libraries are here because they are now needed by the
dynamic libraries.

Developers starting new projects are strongly encouraged to consider
using libelf instead of BFD.

%prep
%setup -q -c -n %{scl_name}
cd %{basesourcename}

%patch01 -p1
%patch02 -p1

# We cannot run autotools as there is an exact requirement of autoconf-2.59.


%build
echo target is %{binutils_target}
cd %{basesourcename}

export CFLAGS="$RPM_OPT_FLAGS"

CARGS=
case %{binutils_target} in i?86*|sparc*|ppc*|s390*|sh*|arm*|aarch64*)
  CARGS="$CARGS --enable-64-bit-bfd"
  ;;
esac

case %{binutils_target} in ia64*)
  CARGS="$CARGS --enable-targets=i386-linux"
  ;;
esac

case %{binutils_target} in ppc*|ppc64*)
  CARGS="$CARGS --enable-targets=spu"
  ;;
esac

case %{binutils_target} in ppc64-*)
  CARGS="$CARGS --enable-targets=powerpc64le-linux"
  ;;
esac

case %{binutils_target} in ppc64le*)
    CARGS="$CARGS --enable-targets=powerpc-linux"
    ;;
esac

case %{binutils_target} in x86_64*|i?86*|arm*|aarch64*)
  CARGS="$CARGS --enable-targets=x86_64-pep"
  ;;
esac

%configure \
  --build=%{_target_platform} --host=%{_target_platform} \
  --target=%{binutils_target} \
  --with-sysroot=/ \
  --enable-ld=%{enable_ld} --enable-gold=%{enable_gold} \
  --with-sysroot=/ \
%if %{enable_shared}
  --enable-shared \
%else
  --disable-shared \
%endif
%if %{enable_deterministic_archives}
  --enable-deterministic-archives \
%else    
  --enable-deterministic-archives=no \
%endif
%if %{enable_lto}
  --enable-lto \
%endif
%if %{default_compress_debug}
  --enable-compressed-debug-sections=all \
%else
  --enable-compressed-debug-sections=none \
%endif
  $CARGS \
  --enable-plugins \
  --with-bugurl=http://bugzilla.oracle.com

%if %{with docs}  
make %{_smp_mflags} tooldir=%{_prefix} all
make %{_smp_mflags} tooldir=%{_prefix} info
%else
make %{_smp_mflags} tooldir=%{_prefix} MAKEINFO=true all
%endif

# Do not use %%check as it is run after %%install where libbfd.so is rebuild
# with -fvisibility=hidden no longer being usable in its shared form.
%if %{without testsuite}
echo ====================TESTSUITE DISABLED=========================
%else
make -k check < /dev/null || :
echo ====================TESTING=========================
cat {gas/testsuite/gas,ld/ld,binutils/binutils}.sum
echo ====================TESTING END=====================
for file in {gas/testsuite/gas,ld/ld,binutils/binutils}.{sum,log}
do
  ln $file binutils-%{_target_platform}-$(basename $file) || :
done
%endif

%install
rm -rf %{buildroot}
cd %{basesourcename}
make install DESTDIR=%{buildroot}
make prefix=%{buildroot}%{_prefix} infodir=%{buildroot}%{_infodir} install-info

# Rebuild libiberty.a with -fPIC.
# Future: Remove it together with its header file, projects should bundle it.
make -C libiberty clean
make CFLAGS="-g -fPIC $RPM_OPT_FLAGS" -C libiberty

# Rebuild libbfd.a with -fPIC.
# Without the hidden visibility the 3rd party shared libraries would export
# the bfd non-stable ABI.
make -C bfd clean
make CFLAGS="-g -fPIC $RPM_OPT_FLAGS -fvisibility=hidden" -C bfd

# Rebuild libopcodes.a with -fPIC.
make -C opcodes clean
make CFLAGS="-g -fPIC $RPM_OPT_FLAGS" -C opcodes

install -m 644 bfd/libbfd.a %{buildroot}%{_libdir}
install -m 644 libiberty/libiberty.a %{buildroot}%{_libdir}
install -m 644 include/libiberty.h %{buildroot}%{_prefix}/include
install -m 644 opcodes/libopcodes.a %{buildroot}%{_libdir}
# Remove Windows/Novell only man pages
rm -f %{buildroot}%{_mandir}/man1/{dlltool,nlmconv,windres,windmc}*

%if %{enable_shared}
chmod +x %{buildroot}%{_libdir}/lib*.so*
%endif

# Prevent programs from linking against libbfd and libopcodes
# dynamically, as they are change far too often.
rm -f %{buildroot}%{_libdir}/lib{bfd,opcodes}.so

# Remove libtool files, which reference the .so libs
rm -f %{buildroot}%{_libdir}/lib{bfd,opcodes}.la

# Sanity check --enable-64-bit-bfd really works.
grep '^#define BFD_ARCH_SIZE 64$' %{buildroot}%{_prefix}/include/bfd.h
# Fix multilib conflicts of generated values by __WORDSIZE-based expressions.
%ifarch %{ix86} x86_64 ppc %{power64} s390 s390x sh3 sh4 sparc sparc64 arm
sed -i -e '/^#include "ansidecl.h"/{p;s~^.*$~#include <bits/wordsize.h>~;}' \
    -e 's/^#define BFD_DEFAULT_TARGET_SIZE \(32\|64\) *$/#define BFD_DEFAULT_TARGET_SIZE __WORDSIZE/' \
    -e 's/^#define BFD_HOST_64BIT_LONG [01] *$/#define BFD_HOST_64BIT_LONG (__WORDSIZE == 64)/' \
    -e 's/^#define BFD_HOST_64_BIT \(long \)\?long *$/#if __WORDSIZE == 32\
#define BFD_HOST_64_BIT long long\
#else\
#define BFD_HOST_64_BIT long\
#endif/' \
    -e 's/^#define BFD_HOST_U_64_BIT unsigned \(long \)\?long *$/#define BFD_HOST_U_64_BIT unsigned BFD_HOST_64_BIT/' \
    %{buildroot}%{_prefix}/include/bfd.h
%endif
touch -r bfd/bfd-in2.h %{buildroot}%{_prefix}/include/bfd.h

# Generate .so linker scripts for dependencies; imported from glibc/Makerules:

# This fragment of linker script gives the OUTPUT_FORMAT statement
# for the configuration we are building.
OUTPUT_FORMAT="\
/* Ensure this .so library will not be used by a link for a different format
   on a multi-architecture system.  */
$(gcc $CFLAGS $LDFLAGS -shared -x c /dev/null -o /dev/null -Wl,--verbose -v 2>&1 | sed -n -f "%{SOURCE1}")"

tee %{buildroot}%{_libdir}/libbfd.so <<EOH
/* GNU ld script */

$OUTPUT_FORMAT

/* The libz dependency is unexpected by legacy build scripts.  */
/* The libdl dependency is for plugin support.  (BZ 889134)  */
INPUT ( %{_libdir}/libbfd.a -liberty -lz -ldl )
EOH

tee %{buildroot}%{_libdir}/libopcodes.so <<EOH
/* GNU ld script */

$OUTPUT_FORMAT

INPUT ( %{_libdir}/libopcodes.a -lbfd )
EOH

# This one comes from gcc
rm -f %{buildroot}%{_infodir}/dir
rm -rf %{buildroot}%{_prefix}/%{binutils_target}

%find_lang binutils
%find_lang opcodes
%find_lang bfd
%find_lang gas
%find_lang gprof
cat opcodes.lang >> binutils.lang
cat bfd.lang >> binutils.lang
cat gas.lang >> binutils.lang
cat gprof.lang >> binutils.lang

if [ -x ld/ld-new ]; then
  %find_lang ld
  cat ld.lang >> binutils.lang
fi
if [ -x gold/ld-new ]; then
  %find_lang gold
  cat gold.lang >> binutils.lang
fi

%clean
rm -rf %{buildroot}

%post
%if "%{enable_gold}" != "no"
%__rm -f %{_bindir}/ld
%{alternatives_cmdline} --install %{_bindir}/ld ld %{_bindir}/ld.bfd %{ld_bfd_priority}
%{alternatives_cmdline} --install %{_bindir}/ld ld %{_bindir}/ld.gold %{ld_gold_priority}
%{alternatives_cmdline} --auto ld
%endif
/sbin/ldconfig
  /sbin/install-info --info-dir=%{_infodir} %{_infodir}/as.info.gz
  /sbin/install-info --info-dir=%{_infodir} %{_infodir}/binutils.info.gz
  /sbin/install-info --info-dir=%{_infodir} %{_infodir}/gprof.info.gz
  /sbin/install-info --info-dir=%{_infodir} %{_infodir}/ld.info.gz
exit 0

%preun
%if "%{enable_gold}" != "no"
if [ $1 = 0 ]; then
  %{alternatives_cmdline} --remove ld %{_bindir}/ld.bfd
  %{alternatives_cmdline} --remove ld %{_bindir}/ld.gold
fi
%endif
if [ $1 = 0 ]; then
  if [ -e %{_infodir}/binutils.info.gz ]
  then
    /sbin/install-info --delete --info-dir=%{_infodir} %{_infodir}/as.info.gz
    /sbin/install-info --delete --info-dir=%{_infodir} %{_infodir}/binutils.info.gz
    /sbin/install-info --delete --info-dir=%{_infodir} %{_infodir}/gprof.info.gz
    /sbin/install-info --delete --info-dir=%{_infodir} %{_infodir}/ld.info.gz
  fi
fi
exit 0

%postun -p /sbin/ldconfig

%files -f  %{basesourcename}/binutils.lang
%defattr(-,root,root,-)
%license %{basesourcename}/COPYING %{basesourcename}/COPYING3 %{basesourcename}/COPYING3.LIB %{basesourcename}/COPYING.LIB
%doc %{basesourcename}/README
%{_bindir}/[!l]*
%ghost %{_bindir}/ld
%{_bindir}/ld.bfd
%if "%{enable_gold}" != "no"
%{_bindir}/ld.gold
%endif
%{_mandir}/man1/*
%if %{enable_shared}
%{_libdir}/lib*.so
%exclude %{_libdir}/libbfd.so
%exclude %{_libdir}/libopcodes.so
%endif
%{_infodir}/[^b]*info*
%{_infodir}/binutils*info*

%files devel
%defattr(-,root,root,-)
%{_prefix}/include/*
%{_libdir}/lib*.a
%{_libdir}/libbfd.so
%{_libdir}/libopcodes.so
%{_infodir}/bfd*info*

%changelog
* Mon Jul 23 2018 Egeyar Bagcioglu <egeyar.bagcioglu@oracle.com> 2.31.1-1
- Initial version of Oracle Arm Toolset 8 binutils based on GNU version 2.31.1.
- Add binutils-testsuite-aarch64-ifunc.patch:
    Fix the false positive ifunc-9 test case
    Disable executable test cases of pr23169 as they are invalid on aarch64
- Tue Oct 09 2018 Egeyar Bagcioglu <egeyar.bagcioglu@oracle.com>
    Backports this already-upstream Oracle patch fixing the bug 28182242:
    1c2649ed7481 - Do not place symbols into a discarded .dynsym

